# Light

This repository contains code and distribution tools for the Light
STS6 vending system.

The `deprecated` directory contains the previous STS4 version of the
code for reference, nothing in that directory is used by the current
version of the code.

## Building

The _Makefile_ provides build targets for bulding for various CPU archutectures.

*Note:* When the `tarball` target executes, the resulting tarball will have a name
Matching the following pattern: `light-{version}.tgz`
It is recommended to rename the tarball by adding in the CPU architecture. For example: `light-{version}-{arch}.tgz`.

### Raspberry Pi 3 and newer

    $ make build-linux-arm7 tarball

### AMD64

    $ make bulid-linux-amd64 tarball

## Production Installation

`make tarball`: build the distribution tarball for the service

1. `tar xzf` the tarball on the target machine
2. Run `./bin/install-light` to create the light user, SystemD service, config file, and install light.

## Key Loading

1. Get the latest public key using the [KMC Automatic Email Gateway](#get-current-public-key).
2. Make sure light.service is stopped.
3. Run `./bin/light-keyreq pubkey.txt > request.txt` (file names are up to you)
4. Email the request file to the [KMC Automatic Email Gateway](#request-new-vkload-file)
6. Run `./bin/light-load SM+Prism+{module ID}_{unique ID}.kmf` to load the response
7. Run `./bin/light-keyprn > keys.csv` to print the loaded keys

## License Loading

1. Get transaction license for the module from Prism
2. Remove all lines before `--VENDCREDIT BEGINS--` and after `--VENDCREDIT ENDS--`
2. run `./bin/light-license transaction.txt` to load the license

## Juice API

Light makes an https request to its configured Juice host to get
commands to execute on the STS6 device. It runs the commands, and
posts responses to back to Juice. See the Prism API docs.

https://api.utiliflex.com/prism-api/#introduction

`src/api/api.go` contains type definitions for the api objects, it's
shared between Light and Juice.

## Device

For the TSM 250, The device `/tty/ACM0` is supported out of the box by
`Linux raspberrypi 4.14.71-v7+ #1145 SMP Fri Sep 21 15:38:35 BST 2018 armv7l GNU/Linux`

At connection time, light runs the `stty` executable to set the device
to `raw` mode (no echo, no line ending translation) which is necessary
for the device to work properly.

The TSM 500 provides a TCP interface on port 5100, and the `DEVICE`
environment variable should be set to the IP address and port for the
device, e.g. `192.168.1.50:5100`. The TSM 500 should not be directly
addressable by the internet, it is unsecured.

## Running tests

You may need to edit the top of the Makefile to set local
configuration (security module IP address, api key).

- `Make lint`: recursively run `golangci-lint` on the code
- `make test-unit`: run unit tests
- `make test-dev`: run integration tests against a running juice api
- `make test-spec`: run the STS 531-10-02 tests

# KMC Automatic Email Gateway

The KMC has an email gateway for automatically getting the current public
key, as well as requesting a new `VKLOAD` file. All emails to this gateway
must be in plain-text and have a properly formatted subject line and body.

All emails should be sent directly to: `stsa_kmc_01@sts.org.za`

## Get current public key

Subject line: `{KMS:GetPublicKey}`

## Request new VKLOAD file

Subject line: `{KMS:GetKlfStsEd2}`<br>
Body: Contents of the `VKLOAD` request file inline (no atachements)
